/*
 * Copyright (c) 2000 Greg Haerr <greg@censoft.com>
 *
 * Null Keyboard Driver
 */
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <wsksymdef.h>
#if 0
#include <keydef.h>
#endif
#define K_VOID 0xffffffff	/* XXX temporary hack */
#include "device.h"

#if 0
#define dprintf printf
#else
#define dprintf(ignored...)
#endif

static int  FBKBD_Open(KBDDEVICE *pkd);
static void FBKBD_Close(void);
static void FBKBD_GetModifierInfo(MWKEYMOD *modifiers, MWKEYMOD *curmodifiers);
static int  FBKBD_Read(MWKEY *buf, MWKEYMOD *modifiers, MWSCANCODE *scancode);
static int  FBKBD_Poll(void);

static int kbd_fd = -1;
static int probekbd_fd = -1;

KBDDEVICE kbddev = {
	FBKBD_Open,
	FBKBD_Close,
	FBKBD_GetModifierInfo,
	FBKBD_Read,
	FBKBD_Poll
};

/*
 * Open the keyboard.
 */
static int
FBKBD_Open(KBDDEVICE *pkd)
{


	dprintf("FBKBD_Open\n");
	kbd_fd = open("/dev/keyboard", 0, O_RDONLY);
	dprintf("kbd_fd:  %d\n", kbd_fd);
	probekbd_fd = open("/dev/keyboard1", 0, O_RDONLY);
	dprintf("probekbd_fd:  %d\n", probekbd_fd);
	if (kbd_fd >= 0 && probekbd_fd >= 0 && errno != ENOENT) {
		return 1;
	}
	return -2;	/* no kbd*/
}

/*
 * Close the keyboard.
 */
static void
FBKBD_Close(void)
{
	dprintf("FBKBD_Close\n");

	if (kbd_fd >= 0) {
		close(kbd_fd);
		kbd_fd = -1;
	}
	if (probekbd_fd >= 0) {
		close(probekbd_fd);
		probekbd_fd = -1;
	}
}

/*
 * Return the possible modifiers for the keyboard.
 */
static  void
FBKBD_GetModifierInfo(MWKEYMOD *modifiers, MWKEYMOD *curmodifiers)
{
    if (modifiers)
        *modifiers = 0;     /* no modifiers available */
    if (curmodifiers)
        *curmodifiers = 0;
}


/* this read itself only returns 16-bits, but the nano-X API returns
 * 32... so we cheat here. */
static unsigned int kbd_button = K_VOID;
#if 0
static unsigned int probekbd_button = K_VOID;
#endif
static size_t howmany;

/*
 * This reads one keystroke from the keyboard, and the current state of
 * the mode keys (ALT, SHIFT, CTRL).  Returns -1 on error, 0 if no data
 * is ready, and 1 if data was read.  This is a non-blocking call.
 */
static int
FBKBD_Read(MWKEY *buf, MWKEYMOD *modifiers, MWSCANCODE *scancode)
{

    #if 0
    int cc;         /* characters read */

    cc = read(kbd_fd, &kbd_button, sizeof(kbd_button));
    if(    (   (kbd_button>>16)==(K_VOID>>16))
            ||
            (howmany <= 0)
    ){
        cc = read(probekbd_fd, &kbd_button, sizeof(kbd_button));
    }

    if (cc > 0 && (kbd_button>>16)!=(K_VOID>>16)) {
        *buf = kbd_button >> 16;
        *modifiers = 0;     /* no modifiers*/
        *scancode = 0;      /* no scancode*/
        kbd_button = K_VOID;
        howmany = 0;
        return 1;   /* Data was read. */
    }
    return 0;   /* No data ready. */
    #endif

    #if 1
	if (howmany > 0 && (kbd_button>>16)!=(K_VOID>>16)){
		*buf = kbd_button >> 16;
		kbd_button = K_VOID;
        *modifiers = 0;     /* no modifiers*/
        *scancode = 0;      /* no scancode*/
		howmany = 0;
		return 1;	/* Data was read. */
	}
	
	return 0;	/* No data ready. */
    #endif
}

/*
 * Poll for keyboard events -- return 1 if there is an event, 0 if not.
 */
static int
FBKBD_Poll(void)
{
  
	howmany = read(kbd_fd, &kbd_button, sizeof(kbd_button));
    if (    (   (kbd_button>>16)==(K_VOID>>16))
            ||
            (howmany <= 0)
    )
    {
	    howmany = read(probekbd_fd, &kbd_button, sizeof(kbd_button));
    }
	return (howmany>0 && (kbd_button>>16)!=(K_VOID>>16)) ? 1 : 0;
}
