/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 * Modified by hippy (rowanmac at internode.on.net) for the uOLED display
 * 
 * Experimental oLED screen driver for Microwindows
 *
 * 15-Jul-2007 rowanmac at internode.on.net
 * Modified this driver into a uOLED driver 
 *
 * 21-Feb-2000 ajr@ecs.soton.ac.uk
 * Stripped down the VGA driver to make a debug driver so that I can debug the
 * rest of the code.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "device.h"
#include "genfont.h"
#include "genmem.h"
#include "oled160.h"

#define	MY_PSD_ADDR	0x01000000	/* dummy video memory addr */


/* uOLED driver entry points*/
static PSD  uOLED160_open(PSD psd);
static void uOLED160_close(PSD psd);
static void uOLED160_getscreeninfo(PSD psd,PMWSCREENINFO psi);;
static void uOLED160_setpalette(PSD psd,int first,int count,MWPALENTRY *pal);
static void uOLED160_drawpixel(PSD psd,MWCOORD x, MWCOORD y, MWPIXELVAL c);
static MWPIXELVAL uOLED160_readpixel(PSD psd,MWCOORD x, MWCOORD y);
static void uOLED160_drawhline(PSD psd,MWCOORD x1, MWCOORD x2, MWCOORD y, MWPIXELVAL c);
static void uOLED160_drawvline(PSD psd,MWCOORD x,MWCOORD y1,MWCOORD y2,MWPIXELVAL c);
static void uOLED160_fillrect(PSD psd,MWCOORD x1,MWCOORD y1,MWCOORD x2,MWCOORD y2,
		MWPIXELVAL c);
void uOLED160_blit(PSD dstpsd, MWCOORD dstx, MWCOORD dsty, MWCOORD w, MWCOORD h,
		PSD srcpsd, MWCOORD srcx, MWCOORD srcy, long op);

// dummy 
static MWBOOL uOLED_mapmemgc(PSD mempsd,MWCOORD w,MWCOORD h,int planes,int bpp,int linelen,
	int size,void *addr);


SCREENDEVICE	scrdev = {
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL,
	uOLED160_open,
	uOLED160_close,
	uOLED160_getscreeninfo,
	uOLED160_setpalette,
	uOLED160_drawpixel,
	uOLED160_readpixel,
	uOLED160_drawhline,
	uOLED160_drawvline,
	uOLED160_fillrect,
	gen_fonts,
	uOLED160_blit,
	NULL,			/* PreSelect*/
	NULL,			/* DrawArea subdriver*/
	NULL,			/* SetIOPermissions*/
	gen_allocatememgc,
	NULL,
	gen_freememgc
};

#define printd(_a)

static PSD
uOLED160_open(PSD psd)
{
	/* init driver variables depending on ega/vga mode*/
	psd->xres = psd->xvirtres = 160;
	psd->yres = psd->yvirtres = 128;
	psd->planes = 1;
	psd->bpp = 16;
	psd->ncolors = 65536;
	psd->pixtype = MWPF_TRUECOLOR565;
	psd->flags = PSF_SCREEN;


        oLED_init("/dev/ttyUSB0");
	
	return psd;
}

static void
uOLED160_close(PSD psd)
{
	printf("uOLED160_close()\n");
	oLED_close();
}

static void
uOLED160_getscreeninfo(PSD psd,PMWSCREENINFO psi)
{
	psi->rows = psd->yvirtres;
	psi->cols = psd->xvirtres;
	psi->planes = psd->planes;
	psi->bpp = psd->bpp;
	psi->ncolors = psd->ncolors;
	psi->pixtype = psd->pixtype;
	psi->fonts = 1;

	/* uOLED 160x128 */
	psi->xdpcm = 53;	/* 27 */
	psi->ydpcm = 53;	/* assumes screen height of 18 cm*/
	printf("uOLED160_getscreeninfo()\n");
}

static void
uOLED160_setpalette(PSD psd,int first,int count,MWPALENTRY *pal)
{
	printf("uOLED160_setpalette(?)\n");
}

static void
uOLED160_drawpixel(PSD psd,MWCOORD x, MWCOORD y, MWPIXELVAL c)
{
	//printf("uOLED160_drawpixel()\n");
	oLED_SetPixel(x,y,c);
}

static MWPIXELVAL
uOLED160_readpixel(PSD psd,MWCOORD x, MWCOORD y)
{
	//printf("uOLED160_readpixel()\n");
	return oLED_GetPixel(x,y);

}

/* Draw horizontal line from x1,y to x2,y including final point*/
static void
uOLED160_drawhline(PSD psd,MWCOORD x1, MWCOORD x2, MWCOORD y, MWPIXELVAL c)
{
	//printf("uOLED160_drawhline()\n");
	oLED_DrawLine(x1, y, x2, y, c);
}

/* Draw a vertical line from x,y1 to x,y2 including final point*/
static void
uOLED160_drawvline(PSD psd,MWCOORD x, MWCOORD y1, MWCOORD y2, MWPIXELVAL c)
{
	//printf("uOLED160_drawvline()\n");
	oLED_DrawLine(x, y1, x, y2, c);
}

static void
uOLED160_fillrect(PSD psd,MWCOORD x1, MWCOORD y1, MWCOORD x2, MWCOORD y2,
	MWPIXELVAL c)
{
//	printf("uOLED160_fillrect()\n");
        oLED_DrawRectangle(x1, y1, x2, y2, c);
}

void uOLED160_blit(PSD dstpsd, MWCOORD dstx, MWCOORD dsty, MWCOORD w, MWCOORD h,
	PSD srcpsd, MWCOORD srcx, MWCOORD srcy, long op)
{
	printf("uOLED160_blit()\n");

// behaves strange 

        /* determine which PSD is the screen */
	if ((srcpsd->addr == MY_PSD_ADDR) && (dstpsd->addr == MY_PSD_ADDR)) {
		/* screen to screen copy */
		oLED_BlkCopy(srcx, srcy, dstx, dsty, w, h);

	} else if (srcpsd->addr == MY_PSD_ADDR) {
		/* screen to off-screen */
		// could save to memory card
	} else if (dstpsd->addr == MY_PSD_ADDR) {
		/* off-screen to screen */
		// could load from memory card
	} else {
		/* error ! */
		printf("oLED_blit with no screen!\n");
		// do it anyway??
		oLED_BlkCopy(srcx, srcy, dstx, dsty, w, h);
	}
}


static MWBOOL
uOLED_mapmemgc(PSD mempsd,MWCOORD w,MWCOORD h,int planes,int bpp,int linelen,
	int size,void *addr)
{
	return 0;
}



